@@ -25,29 +25,11 @@ Runtime checks.
 
 import gtk
 import gst
-import sys
-import instance
-from configure import APPNAME, PYGTK_REQ, PYGST_REQ, GST_REQ, GNONLIN_REQ, PYCAIRO_REQ
 
 from gettext import gettext as _
 
-def initial_checks():
-    """
-    do some basic check
-    If there is a problem, will display the error and exit.
-    """
-    res = _checks()
-    if not res:
-        return
-    message, detail = res
-    # TODO check if we're running graphically
-    dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
-                               buttons=gtk.BUTTONS_OK)
-    dialog.set_markup("<b>"+message+"</b>")
-    dialog.format_secondary_text(detail)
-    dialog.run()
-
-    sys.exit()
+from pitivi.instance import PiTiVi
+from pitivi.configure import APPNAME, PYGTK_REQ, PYGST_REQ, GST_REQ, GNONLIN_REQ, PYCAIRO_REQ
 
 def initiate_videosinks():
     """
@@ -83,8 +65,7 @@ def __try_import__(modulename):
         return False
 
 def _version_to_string(version):
-    import string
-    return string.join([str(x) for x in version], ".")
+    return ".".join([str(x) for x in version])
 
 def _string_to_list(version):
     return [int(x) for x in version.split(".")]
@@ -100,8 +81,8 @@ def check_required_version(modulename):
         if list(gtk.pygtk_version) < _string_to_list(PYGTK_REQ):
             return [PYGTK_REQ, _version_to_string(gtk.pygtk_version)]
     if modulename == "pygst":
-        if list(gst.pygst_version) < _string_to_list(PYGST_REQ):
-            return [PYGST_REQ, _version_to_string(gst.pygst_version)]
+        if list(gst.get_pygst_version()) < _string_to_list(PYGST_REQ):
+            return [PYGST_REQ, _version_to_string(gst.get_pygst_version())]
     if modulename == "cairo":
         import cairo
         if _string_to_list(cairo.cairo_version_string()) < _string_to_list(PYCAIRO_REQ):
@@ -115,9 +96,9 @@ def check_required_version(modulename):
             return [GNONLIN_REQ, gnlver]
     return [None, None]
 
-def _checks():
+def initial_checks():
     reg = gst.registry_get_default()
-    if instance.PiTiVi:
+    if PiTiVi:
         return (_("%s is already running!") % APPNAME,
                 _("An instance of %s is already running in this script.") % APPNAME)
     if not reg.find_plugin("gnonlin"):
@@ -127,8 +108,8 @@ def _checks():
         return (_("Could not find the autodetect plugins!"),
                 _("Make sure you have installed gst-plugins-good and is available in the GStreamer plugin path."))
     if not hasattr(gtk.gdk.Window, 'cairo_create'):
-        return (_("PyGTK doesn't have Cairo support!"),
-                _("Please use a version of the Python bindings for GTK+ built with Cairo support."))
+        return (_("PyGTK doesn't have cairo support!"),
+                _("Please use a version of the GTK+ Python bindings built with cairo support."))
     if not initiate_videosinks():
         return (_("Could not initiate the video output plugins"),
                 _("Make sure you have at least one valid video output sink available (xvimagesink or ximagesink)"))
@@ -136,36 +117,39 @@ def _checks():
         return (_("Could not initiate the audio output plugins"),
                 _("Make sure you have at least one valid audio output sink available (alsasink or osssink)"))
     if not __try_import__("cairo"):
-        return (_("Could not import the Python bindings for cairo"),
-                _("Make sure you have the Python bindings for cairo installed"))
+        return (_("Could not import the cairo Python bindings"),
+                _("Make sure you have the cairo Python bindings installed"))
     if not __try_import__("gtk.glade"):
-        return (_("Could not import the Python bindings for libglade"),
-                _("Make sure you have the Python bindings for libglade installed"))
+        return (_("Could not import the libglade Python bindings"),
+                _("Make sure you have the libglade Python bindings installed"))
     if not __try_import__("goocanvas"):
-        return (_("Could not import the Python bindings for goocanvas"),
-                _("Make sure you have the Python bindings for goocanvas installed"))
+        return (_("Could not import the goocanvas Python bindings"),
+                _("Make sure you have the goocanvas Python bindings installed"))
+    if not __try_import__("xdg"):
+        return (_("Could not import the xdg Python library"),
+                _("Make sure you have the xdg Python library installed"))
     req, inst = check_required_version("gtk")
     if req:
-        return (_("You do not have a recent enough version of the GTK+ Python Bindings (currently %s)") % inst,
-                ("Install a version of the GTK+ Python bindings greater or equal to %s") % req)
+        return (_("You do not have a recent enough version of the GTK+ Python bindings (currently %s)") % inst,
+                _("Install a version of the GTK+ Python bindings greater or equal to %s") % req)
     req, inst = check_required_version("pygst")
     if req:
-        return (_("You do not have a recent enough version of the GStreamer Python Bindings (currently %s)") % inst,
-                ("Install a version of the GStreamer Python bindings greater or equal to %s") % req)
+        return (_("You do not have a recent enough version of the GStreamer Python bindings (currently %s)") % inst,
+                _("Install a version of the GStreamer Python bindings greater or equal to %s") % req)
     req, inst = check_required_version("gst")
     if req:
         return (_("You do not have a recent enough version of GStreamer (currently %s)") % inst,
-                ("Install a version of the GStreamer greater or equal to %s") % req)
+                _("Install a version of the GStreamer greater or equal to %s") % req)
     req, inst = check_required_version("cairo")
     if req:
-        return (_("You do not have a recent enough version of the Cairo Python Bindings (currently %s)") % inst,
-                ("Install a version of the Cairo Python bindings greater or equal to %s") % req)
+        return (_("You do not have a recent enough version of the cairo Python bindings (currently %s)") % inst,
+                _("Install a version of the cairo Python bindings greater or equal to %s") % req)
     req, inst = check_required_version("gnonlin")
     if req:
         return (_("You do not have a recent enough version of the GNonLin GStreamer plugin (currently %s)") % inst,
-                ("Install a version of the GNonLin GStreamer plugin greater or equal to %s") % req)
+                _("Install a version of the GNonLin GStreamer plugin greater or equal to %s") % req)
     if not __try_import__("zope.interface"):
-        return (_("Could not import the Zope Interface module"),
+        return (_("Could not import the Zope interface module"),
                 _("Make sure you have the zope.interface module installed"))
     if not __try_import__("pkg_resources"):
         return (_("Could not import the distutils modules"),
