@@ -24,9 +24,11 @@ GdkPixbuf thumbnail sink
 
 import gobject
 import gst
-import gtk
+import cairo
+import array
+from pitivi.utils import big_to_cairo_alpha_mask, big_to_cairo_red_mask, big_to_cairo_green_mask, big_to_cairo_blue_mask
 
-class PixbufThumbnailSink(gst.BaseSink):
+class CairoSurfaceThumbnailSink(gst.BaseSink):
     """
     GStreamer thumbnailing sink element.
 
@@ -44,14 +46,19 @@ class PixbufThumbnailSink(gst.BaseSink):
                          gst.PAD_SINK,
                          gst.PAD_ALWAYS,
                          gst.Caps("video/x-raw-rgb,"
-                                  "bpp = (int) 24, depth = (int) 24,"
+                                  "bpp = (int) 32, depth = (int) 32,"
                                   "endianness = (int) BIG_ENDIAN,"
-                                  "red_mask = (int) 0x00FF0000, "
-                                  "green_mask = (int) 0x0000FF00, "
-                                  "blue_mask = (int) 0x000000FF, "
+                                  "alpha_mask = (int) %i, "
+                                  "red_mask = (int)   %i, "
+                                  "green_mask = (int) %i, "
+                                  "blue_mask = (int)  %i, "
                                   "width = (int) [ 1, max ], "
                                   "height = (int) [ 1, max ], "
-                                  "framerate = (fraction) [ 0, max ]"))
+                                  "framerate = (fraction) [ 0, max ]"
+                                  % (big_to_cairo_alpha_mask,
+                                     big_to_cairo_red_mask,
+                                     big_to_cairo_green_mask,
+                                     big_to_cairo_blue_mask)))
         )
 
     def __init__(self):
@@ -72,13 +79,17 @@ class PixbufThumbnailSink(gst.BaseSink):
     def do_render(self, buf):
         self.log("buffer %s %d" % (gst.TIME_ARGS(buf.timestamp),
                                    len(buf.data)))
-        pixb = gtk.gdk.pixbuf_new_from_data(buf.data,
-                                            gtk.gdk.COLORSPACE_RGB,
-                                            False,
-                                            8,
-                                            self.width,
-                                            self.height,
-                                            self.width * 3)
+        b = array.array("b")
+        b.fromstring(buf)
+        pixb = cairo.ImageSurface.create_for_data(b,
+            # We don't use FORMAT_ARGB32 because Cairo uses premultiplied
+            # alpha, and gstreamer does not.  Discarding the alpha channel
+            # is not ideal, but the alternative would be to compute the
+            # conversion in python (slow!).
+            cairo.FORMAT_RGB24,
+            self.width,
+            self.height,
+            self.width * 4)
 
         self.emit('thumbnail', pixb, buf.timestamp)
         return gst.FLOW_OK
@@ -86,4 +97,4 @@ class PixbufThumbnailSink(gst.BaseSink):
     def do_preroll(self, buf):
         return self.do_render(buf)
 
-gobject.type_register(PixbufThumbnailSink)
+gobject.type_register(CairoSurfaceThumbnailSink)
