@@ -24,6 +24,8 @@ import gst
 from pitivi.factories.base import OperationFactory
 from pitivi.stream import AudioStream, VideoStream
 
+from gettext import gettext as _
+
 # FIXME: define a proper hierarchy
 class OperationFactoryError(Exception):
     pass
@@ -54,6 +56,57 @@ class TransformFactory(OperationFactory):
     def _requestNewInputStream(self, *args):
         raise OperationFactoryError("TransformFactory doesn't allow request pads")
 
+class EffectFactory (TransformFactory):
+    """
+    Factories that applies an effect on a stream
+    """
+    def __init__ (self, effect, name='', categories=[_("Uncategorized")],
+                  human_name="", description="", icon=None):
+        TransformFactory.__init__(self, name)
+        self.effectname = effect
+        self.categories = categories
+        self.description = description
+        self.human_name = human_name
+        self._setIcon(icon)
+
+    def getHumanName(self):
+        return self.human_name
+
+    def getDescription(self):
+        return self.description
+
+    def getCategories(self):
+        return self.categories
+
+    def _makeBin (self, *args):
+        bin = gst.Bin()
+        fx = gst.element_factory_make(self.effectname, "effect")
+        if isinstance(self.input_streams[0], VideoStream):
+            csp = gst.element_factory_make("ffmpegcolorspace")
+        else:
+            csp = gst.parse_bin_from_description("audioconvert ! audioresample",
+                                                 True)
+
+        bin.add(fx, csp)
+        csp.link(fx)
+
+        bin.add_pad(gst.GhostPad("sink", csp.get_pad("sink")))
+        bin.add_pad(gst.GhostPad("src", fx.get_pad("src")))
+
+        return bin
+
+    def _releaseBin(self, bin):
+        elements = bin.elements()
+        for element in elements.next():
+            del element
+
+
+    def addInputStream(self, stream):
+        return OperationFactory.addInputStream(self, stream)
+
+    def addOutputStream(self, stream):
+        return OperationFactory.addOutputStream(self, stream)
+
 class StreamModifierFactory(TransformFactory):
     """
     Factories that modify the nature/type of a stream.
