@@ -27,10 +27,10 @@ from pitivi.log.loggable import Loggable
 import pitivi.log.log as log
 import gst
 
-STREAM_MATCH_MAXIMUM = 100
 STREAM_MATCH_SAME_CAPS = 60
-STREAM_MATCH_SAME_PAD_NAME = 40
-STREAM_MATCH_COMPATIBLE_CAPS = 30
+STREAM_MATCH_SAME_PAD_NAME = 30
+STREAM_MATCH_COMPATIBLE_CAPS = 20
+STREAM_MATCH_SAME_TYPE = 10
 STREAM_MATCH_NONE = 0
 
 class MultimediaStream(Loggable):
@@ -269,6 +269,10 @@ def find_upstream_demuxer_and_pad(pad):
             continue
 
         element_factory = element.get_factory()
+        if element_factory is None:
+            # python elements don't have a factory
+            return None, None
+
         element_klass = element_factory.get_klass()
 
         if 'Demuxer' in element_klass:
@@ -302,7 +306,11 @@ def get_pad_type(pad):
     if decoder:
         return get_type_from_decoder(decoder)
 
-    return pad.get_caps()[0].get_name().split('/', 1)[0]
+    caps = pad.props.caps
+    if caps is None:
+        caps = pad.get_caps()
+
+    return caps[0].get_name().split('/', 1)[0]
 
 def get_pad_id(pad):
     lst = []
@@ -344,7 +352,7 @@ def get_stream_for_caps(caps, pad=None):
         ret = TextStream(caps, pad_name)
     return ret
 
-def get_stream_for_pad(pad):
+def get_stream_for_pad(pad, store_pad=False):
     log.debug("stream", "pad:%r")
     caps = pad.props.caps
     if caps is None:
@@ -352,6 +360,8 @@ def get_stream_for_pad(pad):
     pad_id = get_pad_id(pad)
     stream = get_stream_for_caps(caps, pad)
     stream.pad_id = pad_id
+    if store_pad:
+        stream.pad = pad
 
     return stream
 
@@ -392,7 +402,7 @@ def get_pads_for_stream(element, stream):
         try:
             ls = [x for x in element.pads() if pad_compatible_stream(x, stream)]
             break
-        except:
+        except TypeError:
             continue
     # FIXME : I'm not 100% certain that checking against the stream pad_name
     # is a good idea ....
@@ -444,6 +454,11 @@ def stream_compare(stream_a, stream_b):
             current_rank += STREAM_MATCH_SAME_CAPS
         elif stream_a.caps.intersect(stream_b.caps):
             current_rank += STREAM_MATCH_COMPATIBLE_CAPS
+        else:
+            name_a = stream_a.caps[0].get_name()
+            name_b = stream_b.caps[0].get_name()
+            if name_a.split("/", 1)[0] == name_b.split("/", 1)[0]:
+                current_rank += STREAM_MATCH_SAME_TYPE
 
     return current_rank
 
