@@ -25,11 +25,13 @@ from bisect import bisect_left
 from pitivi.signalinterface import Signallable
 from pitivi.log.loggable import Loggable
 from pitivi.utils import UNKNOWN_DURATION, closest_item, PropertyChangeTracker
-from pitivi.timeline.track import TrackObject, SourceTrackObject, TrackError
+from pitivi.timeline.track import TrackObject, SourceTrackObject,\
+     TrackEffect, TrackError
 from pitivi.stream import match_stream_groups_map
 from pitivi.utils import start_insort_right, infinity, getPreviousObject, \
         getNextObject
 from pitivi.timeline.gap import Gap, SmallestGapsFinder, invalid_gap
+from pitivi.stream import VideoStream
 
 # Selection modes
 SELECT = 0
@@ -463,11 +465,11 @@ class Selection(Signallable):
 
     def setToObj(self, obj, mode):
         """
-        Convenience method for calling L{setTo} with a single L{TimelineObject}
+        Convenience method for calling L{setSelection} with a single L{TimelineObject}
 
-        @see: L{setTo}
+        @see: L{setSelection}
         """
-        self.setTo(set([obj]), mode)
+        self.setSelection(set([obj]), mode)
 
     def addTimelineObject(self, timeline_object):
         """
@@ -481,9 +483,7 @@ class Selection(Signallable):
         if timeline_object in self.timeline_objects:
             raise TimelineError("TrackObject already in this selection")
 
-    # FIXME : it took me 10 mins to understand what this method does... a more obvious
-    # name would be better :)
-    def setTo(self, objs, mode):
+    def setSelection(self, objs, mode):
         """
         Update the current selection.
 
@@ -532,6 +532,18 @@ class Selection(Signallable):
 
         return set(objects)
 
+    def getSelectedTrackEffects(self):
+        """
+        Returns the list of L{TrackEffect} contained in this selection.
+        """
+        track_effects = []
+        for timeline_object in self.selected:
+            for track in timeline_object.track_objects:
+                if isinstance(track, TrackEffect):
+                    track_effects.append(track)
+
+        return track_effects
+
     def __len__(self):
         return len(self.selected)
 
@@ -915,7 +927,7 @@ class EditingContext(object):
     SLIP_SLIDE = 3
 
     """Encapsulates interactive editing.
-    
+
     This is the base class for interactive editing contexts.
     """
 
@@ -1139,13 +1151,13 @@ class MoveContext(EditingContext):
         self._restoreValues(self.default_originals)
 
     def _overlapsAreTransitions(self, focus, priority):
-        left_gap, right_gap = Gap.findAroundObject(focus)
+        tracks = set(o.track for o in focus.track_objects)
+        left_gap, right_gap = Gap.findAroundObject(focus, tracks=tracks)
 
         focus_end = focus.start + focus.duration
 
         # left_transition
         if left_gap.duration < 0:
-
             left_obj = left_gap.left_object
             left_end = left_obj.start + left_obj.duration
 
@@ -1161,7 +1173,8 @@ class MoveContext(EditingContext):
             #   overlap the previous and previous previous clips
 
             try:
-                prev_prev = self.timeline.getPreviousTimelineObject(left_obj)
+                prev_prev = self.timeline.getPreviousTimelineObject(left_obj,
+                        tracks=tracks)
                 if prev_prev.start + prev_prev.duration > self.focus.start:
                     return False
             except TimelineError:
@@ -1169,7 +1182,6 @@ class MoveContext(EditingContext):
 
         # right transition
         if right_gap.duration < 0:
-
             right_obj = right_gap.right_object
             right_end = right_obj.start + right_obj.duration
 
@@ -1493,6 +1505,7 @@ class Timeline(Signallable, Loggable):
         self.dead_band = 10
         self.edges = TimelineEdges()
         self.property_trackers = {}
+        self._video_caps = None
 
     def addTrack(self, track):
         """
@@ -1506,6 +1519,7 @@ class Timeline(Signallable, Loggable):
             raise TimelineError("Provided track already controlled by the timeline")
 
         self.tracks.append(track)
+        self.updateVideoCaps()
         self._updateDuration()
         track.connect('start-changed', self._trackDurationChangedCb)
         track.connect('duration-changed', self._trackDurationChangedCb)
@@ -1547,6 +1561,15 @@ class Timeline(Signallable, Loggable):
             self.duration = duration
             self.emit('duration-changed', duration)
 
+    def updateVideoCaps (self, caps= None):
+        if caps:
+            self._video_caps = caps
+
+        if self._video_caps:
+            for track in self.tracks:
+                if type(track.stream) is VideoStream:
+                    track.updateCaps(self._video_caps)
+
     def addTimelineObject(self, obj):
         """
         Add the TimelineObject to the Timeline.
@@ -1665,6 +1688,8 @@ class Timeline(Signallable, Loggable):
         timeline_object = TimelineObject(factory)
         start = 0
         for stream, track in stream_map.iteritems():
+            self.debug("Stream: %s, Track: %s, Track duration: %d", str(stream),
+                       str(track), track.duration)
             start = max(start, track.duration)
             track_object = SourceTrackObject(factory, stream)
             track.addTrackObject(track_object)
@@ -1674,6 +1699,63 @@ class Timeline(Signallable, Loggable):
         self.addTimelineObject(timeline_object)
         return timeline_object
 
+    def addEffectFactoryOnObject(self, factory, timeline_objects):
+        """
+        Add EffectTracks corresponding to the effect from the factory to the corresponding
+        L{TimelineObject}s on the timeline
+
+        @param factory: The EffectFactory to add.
+        @type factory: L{EffectFactory}
+        @timeline_objects: The L{TimelineObject}s on whiches you want to add TrackObjects
+                           corresponding to the L{EffectFactory}
+        @type timeline_objects: A C{List} of L{TimelineObject}s
+
+        @raises TimelineError: if the factory doesn't have input or output streams
+        @returns: A list of L{TimelineObject}, L{TrackObject} tuples
+        """
+        #Note: We should maybe be able to handle several streams for effects which
+        #are actually working on audio/video streams
+        self.debug("factory:%r", factory)
+
+        output_stream = factory.getOutputStreams()
+        if not output_stream:
+            raise TimelineError()
+        output_stream = output_stream[0]
+
+        input_stream = factory.getInputStreams()
+        if not input_stream:
+            raise TimelineError()
+        input_stream = input_stream[0]
+
+        track = None
+        for track_ in self.tracks:
+            if type(track_.stream) == type(input_stream):
+                track = track_
+                break
+
+        if track is None:
+          raise TimelineError("There is no Track to add the effect to")
+
+        if not timeline_objects:
+          raise TimelineError("There is no timeline object to add effect to")
+
+        listTimelineObjectTrackObject = []
+        track_object = TrackEffect(factory, input_stream)
+
+        for obj in timeline_objects:
+            copy_track_obj = track_object.copy()
+            track.addTrackObject(copy_track_obj)
+            copy_track_obj.start = obj.start
+            copy_track_obj.duration = obj.duration
+            copy_track_obj.media_duration = obj.media_duration
+            obj.addTrackObject(copy_track_obj)
+            listTimelineObjectTrackObject.append((obj, copy_track_obj))
+
+        self.debug("%s", ["TimelineObject %s => Track object: %s |"\
+                           %(listTo[0], listTo[1])\
+                           for listTo in listTimelineObjectTrackObject])
+        return listTimelineObjectTrackObject
+
     def _getSourceFactoryStreamMap(self, factory):
         # track.stream -> track
         track_stream_to_track_map = dict((track.stream, track)
@@ -1753,9 +1835,9 @@ class Timeline(Signallable, Loggable):
         """
         Update the timeline's selection with the given selection and mode.
 
-        @see: L{Selection.setTo}
+        @see: L{Selection.setSelection}
         """
-        self.selection.setTo(selection, mode)
+        self.selection.setSelection(selection, mode)
 
     def linkSelection(self):
         """
@@ -1823,8 +1905,8 @@ class Timeline(Signallable, Loggable):
             old_track_objects.extend(timeline_object.track_objects)
             self.removeTimelineObject(timeline_object, deep=True)
 
-        self.selection.setTo(old_track_objects, UNSELECT)
-        self.selection.setTo(new_timeline_object.track_objects, SELECT_ADD)
+        self.selection.setSelection(old_track_objects, UNSELECT)
+        self.selection.setSelection(new_timeline_object.track_objects, SELECT_ADD)
 
     def ungroupSelection(self):
         new_track_objects = []
@@ -1832,18 +1914,31 @@ class Timeline(Signallable, Loggable):
             if len(timeline_object.track_objects) == 1:
                 continue
 
-            self.selection.setTo(timeline_object.track_objects, UNSELECT)
+            self.selection.setSelection(timeline_object.track_objects, UNSELECT)
+            n_track_effects = []
+            n_tl_objects = []
 
             for track_object in list(timeline_object.track_objects):
-                new_track_object = track_object.copy()
-                new_timeline_object = TimelineObject(new_track_object.factory)
-                new_timeline_object.addTrackObject(new_track_object)
-                self.addTimelineObject(new_timeline_object)
-                new_track_objects.extend(new_timeline_object.track_objects)
+                if isinstance(track_object, TrackEffect):
+                    n_track_effects.append(track_object)
+                else:
+                    new_track_object = track_object.copy()
+                    new_timeline_object = TimelineObject(new_track_object.factory)
+                    new_timeline_object.addTrackObject(new_track_object)
+                    n_tl_objects.append(new_timeline_object)
+
+            for tl_object in n_tl_objects:
+                for tck_effect in n_track_effects:
+                    if tl_object.track_objects[0].stream_type == tck_effect.stream_type:
+                        self.addEffectFactoryOnObject(tck_effect.factory, [tl_object])
+
+                self.addTimelineObject(tl_object)
+
+            new_track_objects.extend(new_timeline_object.track_objects)
 
             self.removeTimelineObject(timeline_object, deep=True)
 
-        self.selection.setTo(new_track_objects, SELECT_ADD)
+        self.selection.setSelection(new_track_objects, SELECT_ADD)
 
     def deleteSelection(self):
         """
@@ -1852,7 +1947,7 @@ class Timeline(Signallable, Loggable):
         self.unlinkSelection()
         for timeline_object in self.selection:
             self.removeTimelineObject(timeline_object, deep=True)
-        self.selection.setTo(set([]), SELECT)
+        self.selection.setSelection(set([]), SELECT)
 
     def split(self, time):
         """
@@ -1902,6 +1997,7 @@ class Timeline(Signallable, Loggable):
         """
         Unblock internal updates. Use this after calling L{disableUpdates}.
         """
+
         for track in self.tracks:
             track.enableUpdates()
 
@@ -1909,34 +2005,119 @@ class Timeline(Signallable, Loggable):
 
         self.emit("disable-updates", False)
 
-    def getObjsAtTime(self, time_):
-        objects = self.timeline_objects
-        return [obj for obj in objects if
-            (obj.start < time_) and 
-            ((obj.start + obj.duration) > time_)]
+    def getObjsAtTime(self, time):
+        objects = []
+        for obj in self.timeline_objects:
+            if obj.start < time:
+                if (obj.start + obj.duration) > time:
+                    objects.append(obj)
+            else:
+                break
+        return objects
 
     def getObjsAfterObj(self, obj):
         return self.getObjsAfterTime(obj.start + obj.duration)
 
     def getObjsAfterTime(self, target):
-        return [to for to in self.timeline_objects 
-            if to.start >= target]
+        objects = []
+        for i in range(0, len(self.timeline_objects)):
+            if self.timeline_objects[i].start >= target:
+                objects.extend(self.timeline_objects[i:])
+                break
+        return objects
 
     def getObjsBeforeObj(self, obj):
         return self.getObjsBeforeTime(obj.start)
 
     def getObjsBeforeTime(self, target):
-        return [to for to in self.timeline_objects 
-            if to.start + to.duration <=target]
+        objects = []
+        for obj in self.timeline_objects:
+            if obj.start > target:
+                break
+            elif obj.start + obj.duration <= target:
+                objects.append(obj)
+        return objects
 
     def getObjsInRegion(self, start, end, min_priority=0,
         max_priority=4294967295L):
-        objs = []
-        for to in self.timeline_objects:
-            if ((to.start >= start) and
-                ((to.start + to.duration) <= end) and
-                (to.priority >= min_priority) and
-                (to.priority <= max_priority)):
-                objs.append(to)
-        return objs
+        objects = []
+        for obj in self.timeline_objects:
+            if obj.start >= start:
+                if ((obj.start + obj.duration) <= end and
+                obj.priority >= min_priority and
+                obj.priority <= max_priority):
+                    objects.append(obj)
+            elif obj.start > end:
+                break
+        return objects
 
+    def getPrevKeyframe(self, time):
+        tl_objs = []
+
+        # Exclude objects that start after current position.
+        for obj in self.timeline_objects:
+                tl_objs.append(obj)
+                if obj.start > time:
+                    break
+
+        keyframe_positions = self._getKeyframePositions(tl_objs)
+        for n in range(len(keyframe_positions) -1, -1, -1):
+            if keyframe_positions[n] < time:
+                return keyframe_positions[n]
+        return None
+
+    def getNextKeyframe(self, time):
+        tl_objs = []
+
+        # Include from first object whose end is after the current
+        # position onward.
+        for obj in self.timeline_objects:
+            if (obj.start + obj.duration) > time:
+                n = self.timeline_objects.index(obj)
+                tl_objs.extend(self.timeline_objects[n:])
+                break
+
+        keyframe_positions = self._getKeyframePositions(tl_objs)
+        for n in range(0, len(keyframe_positions)):
+                if keyframe_positions[n] > time:
+                    return keyframe_positions[n]
+
+        return None
+
+    def _getKeyframePositions(self, timeline_objects):
+        keyframe_positions = set([])
+        for tl_obj in timeline_objects:
+            first_track = True
+            for track_obj in tl_obj.track_objects:
+                start = track_obj.start
+                in_point = track_obj.in_point
+                if first_track:
+                    keyframe_positions.add(start)
+                    keyframe_positions.add(start + track_obj.duration)
+                    first_track = False
+
+                interpolators = track_obj.getInterpolators()
+                for value in interpolators:
+                    interpolator = track_obj.getInterpolator(value)
+                    keyframes = interpolator.getInteriorKeyframes()
+                    for kf in keyframes:
+                        position_in_obj = kf.getTime() + start - in_point
+                        keyframe_positions.add(position_in_obj)
+        keyframe_positions = list(keyframe_positions)
+        keyframe_positions.sort()
+        return keyframe_positions
+
+    def getObjsToAddEffectTo(self, point, priority):
+        timeline_objects = []
+        if point == -1:
+            for obj in self.timeline_objects:
+                if obj.priority == priority:
+                    timeline_objects.append(obj)
+        else:
+            for obj in self.timeline_objects:
+                if (obj.start <= point and
+                    point <= (obj.start + obj.duration) and\
+                    obj.priority == priority):
+                    timeline_objects.append(obj)
+
+        return timeline_objects
