@@ -19,9 +19,15 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+import gobject
+
 from pitivi.signalinterface import Signallable
 from pitivi.utils import PropertyChangeTracker
 from pitivi.undo import UndoableAction
+from pitivi.timeline.track import TrackEffect
+
+from pitivi.ui.effectsconfiguration import PROPS_TO_IGNORE
+from pitivi.effects import EffectGstElementPropertyChangeTracker
 
 class TimelineObjectPropertyChangeTracker(PropertyChangeTracker):
     # no out-point
@@ -150,6 +156,113 @@ class TimelineObjectRemoved(UndoableAction):
             track.addTrackObject(track_object)
 
         self.timeline.addTimelineObject(self.timeline_object)
+
+        self._undone()
+
+class TrackEffectAdded(UndoableAction):
+    # Note: We have a bug if we just remove the TrackEffect from the timeline
+    # and keep it saved here and then readd it to corresponding timeline (it
+    # freezes everything). So what we are doing is  to free the TrackEffect,
+    # keep its settings here when undoing, and instanciate a new one when
+    # doing again. We have to keep all EffectPropertyChanged object that refers
+    # to the TrackEffect when undoing so we reset theirs gst_element when
+    # doing it again. The way of doing it is the same with TrackEffectRemoved
+    def __init__(self, timeline_object, track_object, properties_watcher):
+        self.timeline_object = timeline_object
+        self.track_object = track_object
+        self.factory = track_object.factory
+        self.effect_props = []
+        self.gnl_obj_props = []
+        self._properties_watcher = properties_watcher
+        self._props_changed = []
+
+    def do(self):
+        timeline = self.timeline_object.timeline
+        tl_obj_track_obj = timeline.addEffectFactoryOnObject(self.factory,
+                                            timeline_objects=[self.timeline_object])
+
+        self.track_object = tl_obj_track_obj[0][1]
+        element = self.track_object.getElement()
+        for prop_name, prop_value in self.effect_props:
+            element.set_property(prop_name, prop_value)
+        for prop_name, prop_value in self.gnl_obj_props:
+            self.track_object.gnl_object.set_property(prop_name, prop_value)
+        for prop_changed in self._props_changed:
+            prop_changed.gst_element = self.track_object.getElement()
+        self._props_changed = []
+
+        self._done()
+
+    def undo(self):
+        element = self.track_object.getElement()
+        props =  gobject.list_properties(element)
+        self.effect_props = [(prop.name, element.get_property(prop.name))
+                              for prop in props
+                              if prop.flags & gobject.PARAM_WRITABLE
+                              and prop.name not in PROPS_TO_IGNORE]
+        gnl_props =  gobject.list_properties(self.track_object.gnl_object)
+        gnl_obj = self.track_object.gnl_object
+        self.gnl_obj_props = [(prop.name, gnl_obj.get_property(prop.name))
+                              for prop in gnl_props
+                              if prop.flags & gobject.PARAM_WRITABLE]
+
+        self.timeline_object.removeTrackObject(self.track_object)
+        self.track_object.track.removeTrackObject(self.track_object)
+        self._props_changed =\
+            self._properties_watcher.getPropChangedFromTrackObj(self.track_object)
+        del self.track_object
+        self.track_object = None
+
+        self._undone()
+
+class TrackEffectRemoved(UndoableAction):
+    def __init__(self, timeline_object, track_object, properties_watcher):
+        self.track_object = track_object
+        self.timeline_object = timeline_object
+        self.factory = track_object.factory
+        self.effect_props = []
+        self.gnl_obj_props = []
+        self._properties_watcher = properties_watcher
+        self._props_changed = []
+
+    def do(self):
+        element = self.track_object.getElement()
+        props =  gobject.list_properties(element)
+        self.effect_props = [(prop.name, element.get_property(prop.name))
+                              for prop in props
+                              if prop.flags & gobject.PARAM_WRITABLE
+                              and prop.name not in PROPS_TO_IGNORE]
+
+        gnl_props =  gobject.list_properties(self.track_object.gnl_object)
+        gnl_obj = self.track_object.gnl_object
+        self.gnl_obj_props = [(prop.name, gnl_obj.get_property(prop.name))
+                              for prop in gnl_props
+                              if prop.flags & gobject.PARAM_WRITABLE]
+
+        self.timeline_object.removeTrackObject(self.track_object)
+        self.track_object.track.removeTrackObject(self.track_object)
+        self._props_changed =\
+            self._properties_watcher.getPropChangedFromTrackObj(self.track_object)
+        del self.track_object
+        self.track_object = None
+
+        self._done()
+
+    def undo(self):
+        timeline = self.timeline_object.timeline
+        tl_obj_track_obj = timeline.addEffectFactoryOnObject(self.factory,
+                                            timeline_objects=[self.timeline_object])
+
+        self.track_object = tl_obj_track_obj[0][1]
+        element = self.track_object.getElement()
+        for prop_name, prop_value in self.effect_props:
+            element.set_property(prop_name, prop_value)
+        for prop_name, prop_value in self.gnl_obj_props:
+            self.track_object.gnl_object.set_property(prop_name, prop_value)
+        for prop_changed in self._props_changed:
+            prop_changed.gst_element = self.track_object.getElement()
+        self._props_changed = []
+
         self._undone()
 
 class InterpolatorKeyframeAdded(UndoableAction):
@@ -200,18 +313,47 @@ class InterpolatorKeyframeChanged(UndoableAction):
         self.keyframe.setTime(time)
         self.keyframe.setValue(value)
 
+class ActivePropertyChanged(UndoableAction):
+    def __init__(self, effect_action, active):
+        self.effect_action = effect_action
+        self.active = not active
+
+    def do(self):
+        self.effect_action.track_object.active = self.active
+        self.active = not self.active
+        self._done()
+
+    def undo(self):
+        self.effect_action.track_object.active = self.active
+        self.active = not self.active
+        self._undone()
+
 class TimelineLogObserver(object):
     timelinePropertyChangedAction = TimelineObjectPropertyChanged
     timelineObjectAddedAction = TimelineObjectAdded
     timelineObjectRemovedAction = TimelineObjectRemoved
+    trackEffectAddAction = TrackEffectAdded
+    trackEffectRemovedAction = TrackEffectRemoved
     interpolatorKeyframeAddedAction = InterpolatorKeyframeAdded
     interpolatorKeyframeRemovedAction = InterpolatorKeyframeRemoved
     interpolatorKeyframeChangedAction = InterpolatorKeyframeChanged
+    activePropertyChangedAction = ActivePropertyChanged
 
     def __init__(self, log):
         self.log = log
         self.timeline_object_property_trackers = {}
         self.interpolator_keyframe_trackers = {}
+        self.effect_properties_tracker = EffectGstElementPropertyChangeTracker(log)
+        self._pipeline = None
+
+    def setPipeline(self, pipeline):
+        self._pipeline = pipeline
+        self.effect_properties_tracker.pipeline = pipeline
+
+    def getPipeline(self):
+        return self._pipeline
+
+    pipeline = property(getPipeline, setPipeline)
 
     def startObserving(self, timeline):
         self._connectToTimeline(timeline)
@@ -256,6 +398,8 @@ class TimelineLogObserver(object):
     def _connectToTrackObject(self, track_object):
         for prop, interpolator in track_object.getInterpolators().itervalues():
             self._connectToInterpolator(interpolator)
+        if isinstance(track_object, TrackEffect):
+            self.effect_properties_tracker.addEffectElement (track_object.getElement())
 
     def _disconnectFromTrackObject(self, track_object):
         for prop, interpolator in track_object.getInterpolators().itervalues():
@@ -293,11 +437,30 @@ class TimelineLogObserver(object):
         self.log.push(action)
 
     def _timelineObjectTrackObjectAddedCb(self, timeline_object, track_object):
-        self._connectToTrackObject(track_object)
+        if isinstance(track_object, TrackEffect):
+            action = self.trackEffectAddAction(timeline_object, track_object,
+                                               self.effect_properties_tracker)
+            #We use the action instead of the track object
+            #because the track_object changes when redoing
+            track_object.connect("active-changed",
+                                 self._trackObjectActiveChangedCb, action)
+            self.log.push(action)
+            element = track_object.getElement()
+            if element:
+                self.effect_properties_tracker.addEffectElement(element)
+        else:
+            self._connectToTrackObject(track_object)
+
 
     def _timelineObjectTrackObjectRemovedCb(self, timeline_object,
-            track_object):
-        self._disconnectFromTrackObject(track_object)
+                                            track_object):
+        if isinstance(track_object, TrackEffect):
+            action = self.trackEffectRemovedAction(timeline_object,
+                                                track_object,
+                                                self.effect_properties_tracker)
+            self.log.push(action)
+        else:
+            self._disconnectFromTrackObject(track_object)
 
     def _interpolatorKeyframeAddedCb(self, track_object, keyframe):
         action = self.interpolatorKeyframeAddedAction(track_object, keyframe)
@@ -308,6 +471,10 @@ class TimelineLogObserver(object):
         action = self.interpolatorKeyframeRemovedAction(track_object, keyframe)
         self.log.push(action)
 
+    def _trackObjectActiveChangedCb(self, track_object, active, add_effect_action):
+        action = self.activePropertyChangedAction(add_effect_action, active)
+        self.log.push(action)
+
     def _interpolatorKeyframeMovedCb(self, tracker, track_object,
             keyframe, old_snapshot, new_snapshot):
         action = self.interpolatorKeyframeChangedAction(track_object,
