@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # PiTiVi , Non-linear video editor
 #
 #       ui/mainwindow.py
@@ -29,8 +30,8 @@ import gtk
 import gobject
 gobject.threads_init()
 import gst
-import gst.pbutils
 from urllib import unquote
+import webbrowser
 
 try:
     import gconf
@@ -40,14 +41,15 @@ else:
     HAVE_GCONF = True
 
 from gettext import gettext as _
+from gtk import RecentManager
 
 from pitivi.log.loggable import Loggable
 
 from pitivi.ui.timeline import Timeline
-from pitivi.ui.projecttabs import ProjectTabs
+from pitivi.ui.basetabs import BaseTabs
 from pitivi.ui.viewer import PitiviViewer
-from pitivi.configure import pitivi_version, APPNAME, get_pixmap_dir, \
-     get_global_pixmap_dir, LIBDIR
+from pitivi.configure import pitivi_version, APPNAME, APPURL, APPMANUALURL, \
+     get_pixmap_dir, LIBDIR
 from pitivi.ui import dnd
 from pitivi.pipeline import Pipeline
 from pitivi.action import ViewAction
@@ -56,7 +58,10 @@ from pitivi.receiver import receiver, handler
 import pitivi.formatters.format as formatter
 from pitivi.sourcelist import SourceListError
 from pitivi.ui.sourcelist import SourceList
-from pitivi.ui.common import beautify_factory
+from pitivi.ui.effectlist import EffectList
+from pitivi.ui.clipproperties import ClipProperties
+from pitivi.ui.common import SPACING
+from pitivi.ui.common import factory_name
 from pitivi.utils import beautify_length
 from pitivi.ui.zoominterface import Zoomable
 
@@ -79,6 +84,10 @@ GlobalSettings.addConfigOption('mainWindowHPanePosition',
     section="main-window",
     key="hpane-position",
     type_=int)
+GlobalSettings.addConfigOption('mainWindowMainHPanePosition',
+    section="main-window",
+    key="main-hpane-position",
+    type_=int)
 GlobalSettings.addConfigOption('mainWindowVPanePosition',
     section="main-window",
     key="vpane-position",
@@ -118,6 +127,11 @@ GlobalSettings.addConfigOption('elementSettingsDialogHeight',
     section='export',
     key='element-settings-dialog-height',
     default = 460)
+GlobalSettings.addConfigSection("effect-configuration")
+GlobalSettings.addConfigOption('effectVPanedPosition',
+    section='effect-configuration',
+    key='effect-vpaned-position',
+    type_=int)
 
 def supported(info):
     return formatter.can_handle_location(info[1])
@@ -179,8 +193,11 @@ class PitiviMainWindow(gtk.Window, Loggable):
         create_stock_icons()
         self._setActions(instance)
         self._createUi(instance)
+
         self.app = instance
+        self.manager = RecentManager()
         self._zoom_duration_changed = False
+        self._missingUriOnLoading = False
 
         self.app.projectManager.connect("new-project-loading",
                 self._projectManagerNewProjectLoadingCb)
@@ -217,8 +234,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         else:
             self.webcam_button.set_sensitive(False)
 
-        self.show()
-
     def showEncodingDialog(self, project, pause=True):
         """
         Shows the L{EncodingDialog} for the given project Timeline.
@@ -252,18 +267,18 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.actions = [
             ("NewProject", gtk.STOCK_NEW, None,
              None, _("Create a new project"), self._newProjectMenuCb),
-            ("OpenProject", gtk.STOCK_OPEN, None,
+            ("OpenProject", gtk.STOCK_OPEN, _("_Open..."),
              None, _("Open an existing project"), self._openProjectCb),
             ("SaveProject", gtk.STOCK_SAVE, None,
              None, _("Save the current project"), self._saveProjectCb),
-            ("SaveProjectAs", gtk.STOCK_SAVE_AS, None,
+            ("SaveProjectAs", gtk.STOCK_SAVE_AS, _("Save _As..."),
              None, _("Save the current project"), self._saveProjectAsCb),
             ("RevertToSavedProject", gtk.STOCK_REVERT_TO_SAVED, None,
-             None, _("Reload the current project"), self._revertToSavedProjectCb),             
+             None, _("Reload the current project"), self._revertToSavedProjectCb),
             ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project Settings"),
              None, _("Edit the project settings"), self._projectSettingsCb),
             ("RenderProject", 'pitivi-render' , _("_Render project"),
-             None, _("Render project"), self._recordCb),
+             None, _("Render project..."), self._recordCb),
             ("Undo", gtk.STOCK_UNDO,
              _("_Undo"),
              "<Ctrl>Z", _("Undo the last operation"), self._undoCb),
@@ -287,6 +302,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
             ("Quit", gtk.STOCK_QUIT, None, None, None, self._quitCb),
             ("About", gtk.STOCK_ABOUT, None, None,
              _("Information about %s") % APPNAME, self._aboutCb),
+            ("UserManual", gtk.STOCK_HELP, _("User manual"),
+             None, None, self._userManualCb),
             ("File", None, _("_File")),
             ("Edit", None, _("_Edit")),
             ("View", None, _("_View")),
@@ -317,6 +334,9 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.actiongroup = gtk.ActionGroup("mainwindow")
         self.actiongroup.add_actions(self.actions)
         self.actiongroup.add_toggle_actions(self.toggleactions)
+        self.undock_action = gtk.Action("WindowizeViewer", _("Undock Viewer"),
+            _("Put the viewer in a separate window"), None)
+        self.actiongroup.add_action(self.undock_action)
 
         # deactivating non-functional actions
         # FIXME : reactivate them
@@ -340,13 +360,13 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 action.set_visible(False)
             elif action_name in [
                 "ProjectSettings", "Quit", "File", "Edit", "Help", "About",
-                "View", "FullScreen", "FullScreenAlternate",
+                "View", "FullScreen", "FullScreenAlternate", "UserManual",
                 "ImportSourcesFolder", "PluginManager", "PlayPause",
                 "Project", "FrameForward", "FrameBackward",
                 "ShowHideMainToolbar", "ShowHideTimelineToolbar", "Library",
                 "Timeline", "Viewer", "FrameForward", "FrameBackward",
                 "SecondForward", "SecondBackward", "EdgeForward",
-                "EdgeBackward", "Preferences"]:
+                "EdgeBackward", "Preferences", "WindowizeViewer"]:
                 action.set_sensitive(True)
             elif action_name in ["NewProject", "SaveProjectAs", "OpenProject"]:
                 if instance.settings.fileSupportEnabled:
@@ -395,39 +415,69 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.timeline = Timeline(instance, self.uimanager)
         self.timeline.project = self.project
 
-        vpaned.pack2(self.timeline, resize=False, shrink=False)
+        vpaned.pack2(self.timeline, resize=True, shrink=False)
         self.timeline.show()
-        hpaned = gtk.HPaned()
-        vpaned.pack1(hpaned, resize=True, shrink=False)
-        hpaned.show()
-        self.projecttabs = ProjectTabs()
+        self.mainhpaned = gtk.HPaned()
+        vpaned.pack1(self.mainhpaned, resize=True, shrink=False)
+
+        self.secondhpaned = gtk.HPaned()
+        self.mainhpaned.pack1(self.secondhpaned, resize=True, shrink=False)
+        self.secondhpaned.show()
+        self.mainhpaned.show()
+
+        self.projecttabs = BaseTabs(instance)
 
         self.sourcelist = SourceList(instance, self.uimanager)
         self.projecttabs.append_page(self.sourcelist, gtk.Label(_("Media Library")))
         self._connectToSourceList()
         self.sourcelist.show()
 
-        hpaned.pack1(self.projecttabs, resize=True, shrink=False)
+        self.effectlist = EffectList(instance, self.uimanager)
+        self.projecttabs.append_page(self.effectlist, gtk.Label(_("Effect Library")))
+        self.effectlist.show()
+
+        self.secondhpaned.pack1(self.projecttabs, resize=True, shrink=False)
         self.projecttabs.show()
 
+        # Actions with key accelerators that will be made unsensitive while
+        # a gtk entry box is used to avoid conflicts.
+        self.sensitive_actions = []
+        for action in self.timeline.playhead_actions:
+            self.sensitive_actions.append(action[0])
+        for action in self.toggleactions:
+            self.sensitive_actions.append(action[0])
+
+        #Clips properties
+        self.propertiestabs = BaseTabs(instance, True)
+        self.clipconfig = ClipProperties(instance, self.uimanager)
+        self.clipconfig.project = self.project
+        self.propertiestabs.append_page(self.clipconfig,
+                                        gtk.Label(_("Effects configurations")))
+        self.clipconfig.show()
+
+        self.secondhpaned.pack2(self.propertiestabs, resize= True, shrink=False)
+        self.propertiestabs.show()
+
         # Viewer
-        self.viewer = PitiviViewer()
+        self.viewer = PitiviViewer(instance, undock_action=self.undock_action)
         # drag and drop
         self.viewer.drag_dest_set(gtk.DEST_DEFAULT_DROP | gtk.DEST_DEFAULT_MOTION,
                            [dnd.FILESOURCE_TUPLE, dnd.URI_TUPLE],
                            gtk.gdk.ACTION_COPY)
         self.viewer.connect("drag_data_received", self._viewerDndDataReceivedCb)
-        hpaned.pack2(self.viewer, resize=False, shrink=False)
-        self.viewer.show()
+        self.mainhpaned.pack2(self.viewer, resize=False, shrink=False)
         self.viewer.connect("expose-event", self._exposeEventCb)
 
         # window and pane position defaults
-        self.hpaned = hpaned
+        self.mainhpaned = self.mainhpaned
+        self.hpaned = self.secondhpaned
         self.vpaned = vpaned
         height = -1
         width = -1
         if self.settings.mainWindowHPanePosition:
             self.hpaned.set_position(self.settings.mainWindowHPanePosition)
+        if self.settings.mainWindowMainHPanePosition:
+            self.mainhpaned.set_position(self.settings.mainWindowMainHPanePosition)
         if self.settings.mainWindowVPanePosition:
             self.vpaned.set_position(self.settings.mainWindowVPanePosition)
         if self.settings.mainWindowWidth:
@@ -450,8 +500,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         vbox.pack_start(ttb, expand=False)
         ttb.show()
 
-        self.show()
-
         if not self.settings.mainWindowShowMainToolbar:
             self.toolbar.props.visible = False
 
@@ -477,21 +525,30 @@ class PitiviMainWindow(gtk.Window, Loggable):
             self.viewer.window.unfullscreen()
             self.is_fullscreen = False
 
-## PlayGround callback
+    #TODO check if it is the way to go
+    def setActionsSensitive(self, action_names, sensitive):
+        """
+        Grab (or release) keyboard letter keys focus/sensitivity
+        for operations such as typing text in an entry.
+        @param action_names: The name of actions we
+                             want to set to sensitive or not, if set to "default"
+                             we use the default actions.
+        @type action_names: A {list} of action names
+        @param sensitive: %True if actions must be sensitive False otherwise
+        @type action_names: C{Bool}
+        """
+        if action_names == "default":
+            action_names = self.sensitive_actions
 
-    def _windowizeViewer(self, button, pane):
-        # FIXME: the viewer can't seem to handle being unparented/reparented
-        pane.remove(self.viewer)
-        window = gtk.Window()
-        window.add(self.viewer)
-        window.connect("destroy", self._reparentViewer, pane)
-        window.resize(200, 200)
-        window.show_all()
+        for action in self.actiongroup.list_actions():
+            if action.get_name() in action_names:
+                action.set_sensitive(sensitive)
 
-    def _reparentViewer(self, window, pane):
-        window.remove(self.viewer)
-        pane.pack2(self.viewer, resize=False, shrink=False)
-        self.viewer.show()
+        if self.timeline:
+            for action_group in self.timeline.ui_manager.get_action_groups():
+                for action in action_group.list_actions():
+                    if action.get_name() in action_names:
+                        action.set_sensitive(sensitive)
 
 ## Missing Plugin Support
 
@@ -525,6 +582,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _saveWindowSettings(self):
         self.settings.mainWindowFullScreen = self.is_fullscreen
         self.settings.mainWindowHPanePosition = self.hpaned.get_position()
+        self.settings.mainWindowMainHPanePosition = self.mainhpaned.get_position()
         self.settings.mainWindowVPanePosition = self.vpaned.get_position()
         mtb = self.actiongroup.get_action("ShowHideMainToolbar")
         ttb = self.actiongroup.get_action("ShowHideTimelineToolbar")
@@ -541,7 +599,90 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.app.projectManager.newBlankProject()
 
     def _openProjectCb(self, unused_action):
+        self.openProject()
 
+    def _saveProjectCb(self, unused_action):
+        if not self.project.uri:
+            self._saveProjectAsCb(unused_action)
+        else:
+            self.app.projectManager.saveProject(self.project, overwrite=True)
+
+    def _saveProjectAsCb(self, unused_action):
+        uri = self._showSaveAsDialog(self.app.current)
+        if uri is not None:
+            return self.app.projectManager.saveProject(self.project, uri, overwrite=True)
+
+        return False
+
+    def _revertToSavedProjectCb(self, unused_action):
+        return self.app.projectManager.revertToSavedProject()
+
+    def _projectSettingsCb(self, unused_action):
+        self.showProjectSettingsDialog()
+
+    def showProjectSettingsDialog(self):
+        from projectsettings import ProjectSettingsDialog
+        ProjectSettingsDialog(self, self.app.current).show()
+
+    def _quitCb(self, unused_action):
+        self._saveWindowSettings()
+        self.app.shutdown()
+
+    def _fullScreenCb(self, unused_action):
+        self.toggleFullScreen()
+
+    def _fullScreenAlternateCb(self, unused_action):
+        self.actiongroup.get_action("FullScreen").activate()
+
+    def _showHideMainToolBar(self, action):
+        self.uimanager.get_widget("/MainToolBar").props.visible = \
+            action.props.active
+
+    def _showHideTimelineToolbar(self, action):
+        self.uimanager.get_widget("/TimelineToolBar").props.visible = \
+            action.props.active
+
+    def _userManualCb(self, unused_action):
+        webbrowser.open_new (APPMANUALURL)
+
+    def _aboutResponseCb(self, dialog, unused_response):
+        dialog.destroy()
+
+    def _showWebsiteCb(self, dialog, uri):
+        webbrowser.open_new(uri)
+
+    def _aboutCb(self, unused_action):
+        abt = gtk.AboutDialog()
+        abt.set_name(APPNAME)
+        abt.set_version("v%s" % pitivi_version)
+        gtk.about_dialog_set_url_hook(self._showWebsiteCb)
+        abt.set_website(APPURL)
+        authors = ["Edward Hervey <bilboed@bilboed.com>",
+                   "Alessandro Decina <alessandro.decina@collabora.co.uk>",
+                   "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)",
+                   "Jean-François Fortin Tam <nekohayo@gmail.com> (UI)",
+                   "Thibault Saunier <thibault.saunier@collabora.com>",
+                   "",
+                   _("Contributors:"),
+                   "Christophe Sauthier <christophe.sauthier@gmail.com> (i18n)",
+                   "Laszlo Pandy <laszlok2@gmail.com> (UI)",
+                   "Ernst Persson  <ernstp@gmail.com>",
+                   "Richard Boulton <richard@tartarus.org>",
+                   "Thibaut Girka <thibaut.girka@free.fr> (UI)",
+                   "Johan Dahlin <jdahlin@async.com.br> (UI)",
+                   "Luca Della Santina <dellasantina@farm.unipi.it>",
+                   "Thijs Vermeir <thijsvermeir@gmail.com>",
+                   "Sarath Lakshman <sarathlakshman@slynux.org>",
+                   "Alex Balut <alexandru.balut@gmail.com>"]
+        abt.set_authors(authors)
+        abt.set_license(_("GNU Lesser General Public License\n"
+                          "See http://www.gnu.org/copyleft/lesser.html for more details"))
+        abt.set_icon_name("pitivi")
+        abt.set_logo_icon_name("pitivi")
+        abt.connect("response", self._aboutResponseCb)
+        abt.show()
+
+    def openProject(self):
         chooser = gtk.FileChooserDialog(_("Open File..."),
             self,
             action=gtk.FILE_CHOOSER_ACTION_OPEN,
@@ -572,81 +713,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         chooser.destroy()
         return True
 
-    def _saveProjectCb(self, unused_action):
-        if not self.project.uri:
-            self._saveProjectAsCb(unused_action)
-        else:
-            self.app.projectManager.saveProject(self.project, overwrite=True)
-
-    def _saveProjectAsCb(self, unused_action):
-        uri = self._showSaveAsDialog(self.app.current)
-        if uri is not None:
-            return self.app.projectManager.saveProject(self.project, uri, overwrite=True)
-
-        return False
-
-    def _revertToSavedProjectCb(self, unused_action):
-        return self.app.projectManager.revertToSavedProject()
-            
-
-    def _projectSettingsCb(self, unused_action):
-        from projectsettings import ProjectSettingsDialog
-        ProjectSettingsDialog(self, self.app.current).show()
-
-    def _quitCb(self, unused_action):
-        self._saveWindowSettings()
-        self.app.shutdown()
-
-    def _fullScreenCb(self, unused_action):
-        self.toggleFullScreen()
-
-    def _fullScreenAlternateCb(self, unused_action):
-        self.actiongroup.get_action("FullScreen").activate()
-
-    def _showHideMainToolBar(self, action):
-        self.uimanager.get_widget("/MainToolBar").props.visible = \
-            action.props.active
-
-    def _showHideTimelineToolbar(self, action):
-        self.uimanager.get_widget("/TimelineToolBar").props.visible = \
-            action.props.active
-
-    def _aboutResponseCb(self, dialog, unused_response):
-        dialog.destroy()
-
-    def _showWebsiteCb(self, dialog, uri):
-        import webbrowser
-        webbrowser.open_new(uri)
-
-    def _aboutCb(self, unused_action):
-        abt = gtk.AboutDialog()
-        abt.set_name(APPNAME)
-        abt.set_version("v%s" % pitivi_version)
-        gtk.about_dialog_set_url_hook(self._showWebsiteCb)
-        abt.set_website("http://www.pitivi.org/")
-        authors = ["Edward Hervey <bilboed@bilboed.com>",
-                   "Alessandro Decina <alessandro.decina@collabora.co.uk>",
-                   "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)",
-                   "",
-                   _("Contributors:"),
-                   "Christophe Sauthier <christophe.sauthier@gmail.com> (i18n)",
-                   "Laszlo Pandy <laszlok2@gmail.com> (UI)",
-                   "Ernst Persson  <ernstp@gmail.com>",
-                   "Richard Boulton <richard@tartarus.org>",
-                   "Thibaut Girka <thibaut.girka@free.fr> (UI)",
-                   "Jeff Fortin <nekohayo@gmail.com> (UI)",
-                   "Johan Dahlin <jdahlin@async.com.br> (UI)",
-                   "Luca Della Santina <dellasantina@farm.unipi.it>",
-                   "Thijs Vermeir <thijsvermeir@gmail.com>",
-                   "Sarath Lakshman <sarathlakshman@slynux.org>"]
-        abt.set_authors(authors)
-        abt.set_license(_("GNU Lesser General Public License\n"
-                          "See http://www.gnu.org/copyleft/lesser.html for more details"))
-        abt.set_icon_name("pitivi")
-        abt.set_logo_icon_name("pitivi")
-        abt.connect("response", self._aboutResponseCb)
-        abt.show()
-
     def _undoCb(self, action):
         self.app.action_log.undo()
 
@@ -715,8 +781,13 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.render_button.set_sensitive(can_render)
         self._syncDoUndo(self.app.action_log)
 
+        if self._missingUriOnLoading:
+            self.app.current.setModificationState(True)
+            self.actiongroup.get_action("SaveProject").set_sensitive(True)
+            self._missingUriOnLoading = False
+
         if project.timeline.duration != 0:
-            self._setBestZoomRatio()
+            self.setBestZoomRatio()
         else:
             self._zoom_duration_changed = True
 
@@ -725,15 +796,21 @@ class PitiviMainWindow(gtk.Window, Loggable):
         # preliminary seek to ensure the project pipeline is configured
         self.project.seeker.seek(0)
 
-    def _setBestZoomRatio(self):
+    def setBestZoomRatio(self, p=0):
+        """Set the zoom level so that the entire timeline is in view."""
         ruler_width = self.timeline.ruler.get_allocation()[2]
-        timeline_duration = self.project.timeline.duration
+        # Add gst.SECOND - 1 to the timeline duration to make sure the
+        # last second of the timeline will be in view.
+        timeline_duration = self.project.timeline.duration + gst.SECOND - 1
+        timeline_duration_s = int(timeline_duration / gst.SECOND)
 
-        ideal_zoom_ratio = ruler_width / float(timeline_duration / gst.SECOND)
+        ideal_zoom_ratio = float(ruler_width) / timeline_duration_s
         nearest_zoom_level = Zoomable.computeZoomLevel(ideal_zoom_ratio)
         Zoomable.setZoomLevel(nearest_zoom_level)
 
     def _projectManagerNewProjectLoadingCb(self, projectManager, uri):
+        if uri != None :
+            self.manager.add_item(uri)
         self.log("A NEW project is being loaded, deactivate UI")
 
     def _projectManagerSaveProjectFailedCb(self, projectManager,
@@ -782,25 +859,23 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 "changes will be lost")
 
         # put the text in a vbox
-        vbox = gtk.VBox(False, 12)
+        vbox = gtk.VBox(False, SPACING*2)
         vbox.pack_start(primary, expand=True, fill=True)
         vbox.pack_start(secondary, expand=True, fill=True)
 
         # make the [[image] text] hbox
         image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
                gtk.ICON_SIZE_DIALOG)
-        hbox = gtk.HBox(False, 12)
+        hbox = gtk.HBox(False, SPACING*2)
         hbox.pack_start(image, expand=False)
         hbox.pack_start(vbox, expand=True, fill=True)
         action_area = dialog.get_action_area()
-        # FIXME: find out where this "6" comes from. It's needed to align our
-        # hbox with the action area button box
-        hbox.set_border_width(6)
+        hbox.set_border_width(SPACING)
 
         # stuff the hbox in the dialog
         content_area = dialog.get_content_area()
         content_area.pack_start(hbox, expand=True, fill=True)
-        content_area.set_spacing(14)
+        content_area.set_spacing(SPACING*2)
         hbox.show_all()
 
         response = dialog.run()
@@ -838,7 +913,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
                                         gtk.STOCK_REVERT_TO_SAVED, gtk.RESPONSE_YES)
             dialog.set_title(_("Revert to saved project"))
             dialog.set_resizable(False)
-            dialog.set_property("secondary-text", 
+            dialog.set_property("secondary-text",
                                             _("All unsaved changes will be lost.")
                                         )
             dialog.set_default_response(gtk.RESPONSE_NO)
@@ -847,19 +922,18 @@ class PitiviMainWindow(gtk.Window, Loggable):
             if response <> gtk.RESPONSE_YES:
                 return False
         return True
-        
+
 
     def _projectManagerNewProjectFailedCb(self, projectManager, uri, exception):
-        # ungrey UI
+        project_filename = unquote(uri.split("/")[-1])
         dialog = gtk.MessageDialog(self,
             gtk.DIALOG_MODAL,
             gtk.MESSAGE_ERROR,
             gtk.BUTTONS_OK,
-            _("PiTiVi is unable to load file \"%s\"") %
-                uri)
+            _('Unable to load project "%s"') % project_filename)
         dialog.set_icon_name("pitivi")
         dialog.set_title(_("Error Loading File"))
-        dialog.set_property("secondary-text", str(exception))
+        dialog.set_property("secondary-text", unquote(str(exception)))
         dialog.run()
         dialog.destroy()
         self.set_sensitive(True)
@@ -871,16 +945,16 @@ class PitiviMainWindow(gtk.Window, Loggable):
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
         dialog.set_icon_name("pitivi")
-        dialog.set_border_width(12)
-        dialog.get_content_area().set_spacing(6)
+        dialog.set_border_width(SPACING*2)
+        dialog.get_content_area().set_spacing(SPACING)
 
-        text = _("The following file has moved, please tell PiTiVi where to find it.") + \
-            "\n\n" + beautify_factory(factory) + "\n" + \
-            "<b>%s</b>" % _("Duration:") + beautify_length(factory.duration)
+        text = _('The following file has moved: "<b>%s</b>" (duration: %s)\
+                \nPlease specify its new location:' \
+                % (factory_name(factory), beautify_length(factory.duration)))
+        # TODO: display the filesize to help the user identify the file
 
         label = gtk.Label()
         label.set_markup(text)
-        label.set_justify(gtk.JUSTIFY_CENTER)
         dialog.get_content_area().pack_start(label, False, False)
         label.show()
 
@@ -890,16 +964,19 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.get_content_area().pack_start(chooser, True, True)
         chooser.show()
 
-        dialog.set_size_request(640, 480)
+        # If the window is too big, the window manager will resize it so that
+        # it fits on the screen.
+        dialog.set_default_size(1024, 1000)
         response = dialog.run()
 
         if response == gtk.RESPONSE_OK:
-            self.log("User chose a URI to save project to")
+            self.log("User chose a new URI for the missing file")
             new = chooser.get_uri()
             if new:
                 formatter.addMapping(uri, unquote(new))
+                self._missingUriOnLoading = True
         else:
-            self.log("User didn't choose a URI to save project to")
+            self.log("User didn't choose a URI for the missing file")
             # FIXME: not calling addMapping doesn't keep the formatter from
             # re-emitting the same signal. How do we get out of this
             # situation?
@@ -961,6 +1038,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
             self.project_timeline = self.project.timeline
             if self.timeline:
                 self.timeline.project = self.project
+                self.clipconfig.project = self.project
+                self.app.timelineLogObserver.pipeline = self.project.pipeline
 
     project = receiver(_setProject)
 
@@ -1007,7 +1086,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         if duration > 0:
             sensitive = True
             if self._zoom_duration_changed:
-                self._setBestZoomRatio()
+                self.setBestZoomRatio()
                 self._zoom_duration_changed = False
         else:
             sensitive = False
