@@ -109,8 +109,10 @@ class Zoomable(object):
 
     @classmethod
     def computeZoomLevel(cls, ratio):
-        return int((((ratio - cls.min_zoom) / cls.zoom_range) ** (1.0/3.0)) *
-            cls.zoom_steps)
+        return int((
+            (max(0, ratio - cls.min_zoom) /
+                cls.zoom_range) ** (1.0/3.0)) *
+                    cls.zoom_steps)
 
     @classmethod
     def pixelToNs(cls, pixel):
@@ -119,6 +121,14 @@ class Zoomable(object):
         """
         return long(pixel * gst.SECOND / cls.zoomratio)
 
+    @classmethod
+    def pixelToNsAt(cls, pixel, ratio):
+        """
+        Returns the pixel equivalent in nanoseconds according to the zoomratio
+        """
+        return long(pixel * gst.SECOND / ratio)
+
+
     @classmethod
     def nsToPixel(cls, duration):
         """
