@@ -27,6 +27,7 @@ import sys
 import gobject
 import gst, bisect
 import os
+import struct
 from pitivi.signalinterface import Signallable
 import pitivi.log.log as log
 from gettext import ngettext
@@ -55,13 +56,11 @@ def time_to_string(value):
     sec = sec % 60
     hours = mins / 60
     mins = mins % 60
-    return "%02d:%02d:%02d.%03d" % (hours, mins, sec, ms)
+    return "%01d:%02d:%02d.%03d" % (hours, mins, sec, ms)
 
 def beautify_length(length):
     """
     Converts the given time in nanoseconds to a human readable string
-
-    Format HHhMMmSSs
     """
     sec = length / gst.SECOND
     mins = sec / 60
@@ -81,6 +80,29 @@ def beautify_length(length):
 
     return ", ".join(parts)
 
+def beautify_ETA(length):
+    """
+    Converts the given time in nanoseconds to a fuzzy estimate,
+    intended for progress ETAs, not to indicate a clip's duration.
+    """
+    sec = length / gst.SECOND
+    mins = sec / 60
+    sec = sec % 60
+    hours = mins / 60
+    mins = mins % 60
+
+    parts = []
+    if hours:
+        parts.append(ngettext("%d hour", "%d hours", hours) % hours)
+
+    if mins:
+        parts.append(ngettext("%d minute", "%d minutes", mins) % mins)
+
+    if not hours and mins < 2 and sec:
+        parts.append(ngettext("%d second", "%d seconds", sec) % sec)
+
+    return ", ".join(parts)
+
 def bin_contains(bin, element):
     """ Returns True if the bin contains the given element, the search is recursive """
     if not isinstance(bin, gst.Bin):
@@ -189,8 +211,9 @@ def data_probe(pad, data, section=""):
 def linkDynamic(element, target):
 
     def pad_added(bin, pad, target):
-        if target.get_compatible_pad(pad):
-            element.link(target)
+        compatpad = target.get_compatible_pad(pad)
+        if compatpad:
+            pad.link_full(compatpad, gst.PAD_LINK_CHECK_NOTHING)
     element.connect("pad-added", pad_added, target)
 
 def element_make_many(*args):
@@ -497,3 +520,11 @@ def profile(func, profiler_filename="result.prof"):
 
 def formatPercent(value):
     return "%3d%%" % (value * 100)
+
+
+native_endianness = struct.pack('=I', 0x34333231)
+
+big_to_cairo_alpha_mask = struct.unpack('=i', '\xFF\x00\x00\x00')[0]
+big_to_cairo_red_mask = struct.unpack('=i', '\x00\xFF\x00\x00')[0]
+big_to_cairo_green_mask = struct.unpack('=i', '\x00\x00\xFF\x00')[0]
+big_to_cairo_blue_mask = struct.unpack('=i', '\x00\x00\x00\xFF')[0]
